﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectMotorboat.CollectionGenericObjects;

public class ListGenericObjects<T> : ICollectionGenericObjects<T>

where T : class
{
    private readonly List<T?> _collection;

    public CollectionType GetCollectionType => CollectionType.List;

    private int _maxCount;

    public int Count => _collection.Count;

    public int MaxCount
    {
        get
        {
            return Count;
        }
        set
        {
            if (value > 0)
            {
                _maxCount = value;
            }
        }
    }

    public ListGenericObjects()
    {
        _collection = new();
    }

    public T? Get(int position)
    {
        if (position >= Count || position < 0) return null;
        return _collection[position];
    }

    public int Insert(T obj)
    {
        if (Count + 1 > _maxCount) return -1;
        _collection.Add(obj);
        return Count;
    }

    public int Insert(T obj, int position)
    {
        if (Count + 1 > _maxCount) return -1;
        if (position < 0 || position > Count) return -1;
        _collection.Insert(position, obj);
        return 1;
    }

    public T? Remove(int position)
    {
        if (position < 0 || position > Count) return null;
        T? pos = _collection[position];
        _collection.RemoveAt(position);
        return pos;
    }

    public IEnumerable<T> GetItems()
    {
        for (int i = 0; i < Count; ++i)
        {
            yield return _collection[i];
        }
    }
}